# 库安装出错因为依赖被占用时，需要先卸载的库列表
UNINSTALL_LIST_MAP = {
    'numpy': ['numpy'],
    'pandas': ['numpy', 'pandas'],
    'pgzero': ['numpy', 'pgzero'],
    'scipy': ['numpy','scipy'],
    'scikit-learn': ['numpy', 'scipy', 'scikit-learn'],
    'seaborn': ['numpy', 'scipy', 'matplotlib', 'pandas', 'seaborn'],
    'openpyxl': ['numpy', 'openpyxl'],
    'xes-lib': ['numpy', 'xes-lib'],
    'imageio':['numpy', 'imageio'],
    "matplotlib":['numpy','matplotlib']
}

ERR_TYPE_MSG_MAP = {
    "PermissionError": ['PermissionError', 'FileExistsError: [WinError 183] 当文件已存在时，无法创建该文件'],
    'NoSpace': ['[Errno 28] No space left on device']
}


def fn_judge_error(msg, module_name):
    '''
    根据stderr的字符串返回是否
    @param msg stderr的字符串
    @return 返回一个成功对象 { "status": "ok" }或失败对象 { "status": "err", 'msg':'PermissionError', "uninstall_list": ['numpy']}
    注意：这里的uninstall_list中的库需要先卸载，再安装目标库
    '''

    check_result = {
        "status": "ok",

    }

    for err_type in ERR_TYPE_MSG_MAP:
        for err_msg in ERR_TYPE_MSG_MAP[err_type]:
            if err_msg in msg:
                print("触发了失败报错", err_type, err_msg)
                check_result["status"] = "err"
                check_result["msg"] = err_msg
                if module_name in UNINSTALL_LIST_MAP:
                    check_result['uninstall_list'] = UNINSTALL_LIST_MAP[module_name]
                else:
                    check_result['uninstall_list'] = []
                return check_result

    return check_result



from xesrepair.common import PYTHONW_EXE, USER_LIB_PATH, create_python_process
def check():
    try:
        import numpy 
        print(numpy.__version__)
        return {"state": True}
    except Exception as e:
        print(e)
        return {"state": False, "reason": str(e)}

def repair():
    check_res = check()
    if not check_res["state"]:
        # 先卸载
        uninstall_args = [PYTHONW_EXE, "-m", "pip", "uninstall", "-y", "numpy"]
        create_python_process(uninstall_args)
        # 再安装
        install_args = [PYTHONW_EXE, "-m", "pip", "install",'--target', USER_LIB_PATH, 'numpy', '--no-cache-dir', '--no-warn-script-location', "--index-url", 'https://mirrors.aliyun.com/pypi/simple/', "--upgrade" ]
        create_python_process(install_args)
        check_res = check()
    return check_res



import sys
import subprocess
import locale
import os

PYTHONW_EXE = sys.executable.replace(
    "xes_py_helper.exe", "pythonw.exe").replace("python.exe", "pythonw.exe")

def get_user_libs_path():
    '''
    :return 获取用户目录的学而思直播地址
    '''
    user_en = locale.getpreferredencoding()
    if user_en == "cp936":
        user_lib_path = os.path.expanduser(r"~\学而思直播\code\site-packages")
    else:
        user_lib_path = os.path.expanduser(r"~\xescode\site-packages")
    return user_lib_path

USER_LIB_PATH = get_user_libs_path()

def create_python_process(args):
    try:
        process_res = subprocess.check_output(args)
        return {
            "state": True,
            "output": process_res
        }
    except Exception as e: 
        print(e)
        return {
            "state": False,
            "output": str(e)
        }




TEMPLATE_LIST = [
    {
        "keyword":"模板爬虫",
        "code_block": '''import requests
import bs4
import time
head = {
    "User-Agent": "Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.8.1.11) Gecko/20071127 Firefox/2.0.0.11"
}# 设置请求头，模拟浏览器
#TODO 修改下方需要爬取的网址
url = "https://static0.xesimg.com/pythonweb/dogs.html"
# 请求网页
res = requests.get(url, headers=head)
res.encoding = res.apparent_encoding  # 设置编码格式
# print(res)  # 查看状态码
# print(res.text)  # 查看网页HTML代码
soup = bs4.BeautifulSoup(res.text, "lxml")  # 解析网页
# 选取数据
#TODO 修改下方爬取条件中的标签名、属性
tags = soup.find_all("div", class_="tit")

# 展示结果
for t in tags:
    print(t.text)
    
    #TODO 取消下面代码的注释，可以保存内容到txt中
    # with open("内容.txt", "a", encoding="UTF-8") as file:
    # #TODO 修改要写入的内容
    #     file.write(t.text + "\n")
'''
    }
    
]



#! python3
# _*_ coding:utf-8 _*_
'''
    添加系统环境变量，即为键Path追加值(路径)
'''

import winreg
import sys
import ctypes


def is_admin():
    '''获取管理员权限'''
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False

def append_Path(value, type=winreg.REG_EXPAND_SZ, keyname='Path'):
	'''默认为键Path追加值(路径)'''
	if is_admin():	# 以管理员身份运行以下代码
        # 连接注册表根键HKEY_LOCAL_MACHINE
		regRoot = winreg.ConnectRegistry(None, winreg.HKEY_LOCAL_MACHINE)
		subDir = r"SYSTEM\CurrentControlSet\Control\Session Manager\Environment"
        # 只读方式打开注册表
		key_read = winreg.OpenKey(regRoot, subDir)
		count = winreg.QueryInfoKey(key_read)[1]  # 获取该目录下所有键的个数(0-下属键个数;1-当前键值个数)
		for i in range(count):
			name,values,type_ = winreg.EnumValue(key_read, i)
			values = values.replace(value, "")
			if name.lower() == keyname.lower():
				if values[-1] == ';':
					values += value
				else:
					values += f';{value}'
				
                # 以只写方式打开注册表
				key_write = winreg.OpenKey(regRoot, subDir, 0, winreg.KEY_WRITE)
                # 追加值
				winreg.SetValueEx(key_write, name, 0, type, values)
				winreg.CloseKey(key_write)
				winreg.CloseKey(key_read)
	else:
		if sys.version_info[0] == 3:
			ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, __file__, None, 1)

def fn_repair_dll_not_found():
	try:
		user_path = sys.executable.replace("\\python.exe", ";").replace("\\pythonw.exe", ";")
		print("运行出现的弹窗请选择【是】")
		append_Path(user_path)
	except Exception as e:
		print(e)
	print("DLL修复完成!\n请重启助手运行代码!")

  



# 特殊配置代码列表，每个字典中包含检测字段 check_code 和 新增代码 add_code
SPECIAL_MAP_ARR = [
    {
        'check_code': 'pgzrun',
        'add_code': '''from pgzero.actor import Actor
from pgzero.rect import Rect
from pgzero.loaders import sounds, images
from pgzero import music, tone
from pgzero.clock import clock
from pgzero.builtins import keymods  # 似乎没有作用
from pgzero.constants import mouse
from pgzero.animation import animate
from pgzero.keyboard import keys, Keyboard, keyboard
from pgzero.screen import Screen, screen
keyboard: Keyboard  # 类型标注
screen: Screen  # 类型标注
'''},
    {
        'check_code': 'df.',
        'add_code': '''import pandas
df:pandas.DataFrame
'''}]

preload_arr = [
    {
        "source": '''import pandas
pandas.''',
        "row": 2,
        "column": 7
    },
    {
        "source": '''import sklearn
sklearn.''',
        "row": 2,
        "column": 8
    },
    {
        "source": '''import numpy
numpy.''',
        "row": 2,
        "column": 6
    }
]

# 补全请求超时限制
OUT_TIME_SECONDS = 10
# 补全黑名单，相应字符后不补全,中文字符
BLACK_LIST = ["！", "$", "￥", "（", "）",")", "-", "【", "】", "、", ":",";","；", "’", ",", "《", "，","。", "》", "？"]
EMPTY_SOURCE = ("a=1", 1, 3) #这种情况下获得的补全为空
from xesrepair.complete_template import TEMPLATE_LIST


class completeFilter:
    def __init__(self):
        pass

    def set_jedi_config(self):
        import jedi
        # jedi.settings.auto_import_modules = ['gi']
        jedi.inference.recursion.recursion_limit = 4
        jedi.inference.recursion.total_function_execution_limit = 6
        jedi.inference.recursion.per_function_execution_limit = 3
        jedi.inference.recursion.per_function_recursion_limit = 2
        # 设置加括号未生效
        # jedi.settings.add_bracket_after_function = True

    def before(self, source, row, column):
        '''
        :功能：对补全代码做前置处理
        :param special_map_arr: 特殊代码匹配的数组
        :param source: 要检测的代码
        :param row: 待补全位置所在行数
        :param column: 当前行待补全位置前的字符个数
        :return 检测特殊代码后重新返回新参数
        '''
        # 1、检测补全位置前是否是黑名单字符
        source, row, column = self.filter_black_char(source, row,column)

        # 2.添加导库语句
        for smap in SPECIAL_MAP_ARR:
            if smap['check_code'] in source:
                source = smap['add_code'] + source
                add_rows = smap['add_code'].count('\n')
                return (source, row+add_rows, column)
        
        return (source, row, column)
    
    def filter_black_char(self,source, row, column):
        '''
        过滤黑名单字符
        '''
        lines = source.split("\n")
        prev_char = lines[row-1][column-1]
        if prev_char in BLACK_LIST:
            return EMPTY_SOURCE
        return source, row, column
    def handle_template(self, source):
        '''
        处理模板
        :return 是否存在模板代码
        '''
        last_line = source.split("\n")[-1]
        for template in TEMPLATE_LIST:
            if last_line == template["keyword"]:
                return [
                    {
                        "meta": "code_block",
                        "caption": template["keyword"],
                        "score": 0,
                        "lang": "python",
                        "value":template["code_block"]
                    }
                ] 
        return None

    def after(self, origin_data, source, row, column):
        '''
        :功能：对补全代码提示做后置处理
        :param origin_data: 初步获得的代码提示数组
        :return 处理过的前端提示
        '''
        template_intelligence = self.handle_template(source)
        if template_intelligence != None:
            return template_intelligence
        target_data = []
        max_index = len(origin_data) - 1
        has_brackets_flag = self.has_brackets(source, row, column)
        for n in origin_data:
            # 如果值的首尾有引号，但是首尾不匹配，那么去掉首尾的引号
            name = n["name"]
            max_index = len(name)-1
            quotation_arr = ['"',  "'"]
            if (name[0] in quotation_arr or name[len(name)-1] in quotation_arr) and name[max_index] != name[0]:
                name = name.strip('"').strip("'")
            if n["type"] == "function" and has_brackets_flag:
                name = name+"()"
            i = {
                "meta": n["type"],
                "caption": name,
                "value": name,
                "score": 0,
                "lang": "python"
            }
            target_data.append(i)
        # 对默认情况不处理
        return target_data
    def has_brackets(self, source, row, column):
        '''判断函数是否加括号
        :retutn bool 是否加括号
        '''
        lines = source.split("\n")
        line = lines[row-1]
        right = line[column:]
        right = right.strip()
        return not right.startswith('(') 


# 用作单例
completeFilterObj = completeFilter()



# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages
setup(name='xesrepair',
      version='0.2.5',
      description='学而思编程助手专属工具库',
      classifiers=[
          'Programming Language :: Python',
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
      ],
      url='https://www.python.org/',
      install_requires=[],
      author='zhushaodong123',
      author_email='',
      license='NEU',
      packages=find_packages(),
      package_data={"": ["*.pyc"]},
      zip_safe=True
      )



import imp
from xesrepair.common import PYTHONW_EXE, USER_LIB_PATH, create_python_process
import subprocess
import sys
import importlib
PACKAGE_NAME = 'xesrepair'
name = PACKAGE_NAME
import platform

SHOW_BUTTON = True  # 是否在学而思编程助手的托盘处进行清理
__version__ = "0.2.5"


class RepairHandler:
    def __init__(self):
        self._modules = ['numpy', 'imageio']  # 待检查的库

    def check_common(self, name):
        try:
            check_module = importlib.import_module(name)
            # 更新
            importlib.reload(sys.modules[name])
            # except Exception as e:
            #     print(e)
            print(check_module.__version__)
            return {"state": True}
        except Exception as e:
            print("check error:"+str(e))
            return {"state": False, "reason": str(e)}

    def repair_common(self, name):
        '''
        :return 修复后再次检测的结果
        '''
        check_res = self.check_common(name)
        if check_res['state']:
            return check_res

        # 通用修复
        # 先卸载
        uninstall_args = [PYTHONW_EXE, "-m", "pip", "uninstall", "-y", name]
        create_python_process(uninstall_args)
        # 再安装
        install_args = [PYTHONW_EXE, "-m", "pip", "install", '--target', USER_LIB_PATH, name, '--no-cache-dir',
                        '--no-warn-script-location', "--index-url", 'https://mirrors.aliyun.com/pypi/simple/', "--upgrade"]
        create_python_process(install_args)

        # 修复后再次检测验证有没有修复成功
        check_res = self.check_common(name)
        return check_res

    def run(self):
        total_check_res = {"state": True}
        # for module in self._modules:
        #     try:
        #         check_module = importlib.import_module(
        #             PACKAGE_NAME+".repair_"+module)
        #         single_check_res = check_module.repair()
        #     except Exception as e:
        #         print(module + ".run() error:"+str(e))
        #         single_check_res = self.repair_common(module)
        #     finally:
        #         # 存储每个模块修复的结果
        #         total_check_res[module] = single_check_res
        #         total_check_res["state"] = total_check_res["state"] and single_check_res["state"]
        if platform.system() == "Windows":
            try:
                from xesrepair.repair_dll import fn_repair_dll_not_found
                fn_repair_dll_not_found()
            except Exception as e:
                total_check_res = {"state": False, "msg": str(e)}
        return total_check_res
